# !/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Agencia
# Module:      librep.py
# Purpose:     Módulo para la reproducción de ficheros de audio wav.
# Language:    Python 2.6
# Date:        22-Nov-2011.
# Ver.:        22-Nov-2011.
# Author:    Rafael J. García Perdigón
#            Ismael Campos Suárez
#            José Carlos Martín
#            Isabel Aparicio Pérez  
#            José L. Redrejo Rodríguez
# Copyright:  2011 - Rafael J. García Perdigón
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################

import wave
import pyaudio
import time
import os

try:
    import mad
except:
    pass #sin soporte de mp3 en linux

    
import threading
import gobject
gobject.threads_init()

dir_tmp=os.path.join(os.path.expanduser("~") ,"tmp")




class audio(threading.Thread):
    """Clase que se encarga del funcionamiento completo del reproductor"""
    
    def __init__(self,fic):
        
        self.fin = False
        self.fichero = fic
        self.mp3=(os.path.splitext(fic)[1].lower()=='.mp3') #con un mp3 en lugar de un wav
        self.mf=None
        
        threading.Thread.__init__(self)
        
        self.data = []
        self.CHUNK= 1024        
        self.stream_out = None
        self.wave=None
        self.status="stop"
        self.posicion=0
        self.direccion="dere"
        self.velocidad=1
        self.unlock=False
        self.online=False
        
        self.p = pyaudio.PyAudio()
        self.reader()
        self.create_stream_out()
        self.start()        
        

    def reader(self): 
        """Convierte la onda de audio, leída del stream creado 
        anteriormente, a una lista de elementos"""               
        data=[]    

        try:            
            ruta = os.path.join(dir_tmp, self.fichero)                       
            if self.mp3:
                self.mf = mad.MadFile(ruta)
            else:
                self.wave=wave.open(ruta, 'rb')

        except Exception,error:
            print error
            self.limpiar()
 
            
        if  self.mp3: 
            data= self.mf.read(self.CHUNK)
            while data:
                self.data.append(data)
                data= self.mf.read(self.CHUNK)
        else:
            data = self.wave.readframes(self.CHUNK)
            while data != '':
                self.data.append(data)
                data = self.wave.readframes(self.CHUNK)
        
        print "Reader Creado..."
        
    def create_stream_out(self):
        """Crea un stream de audio a partir del fichero .wav"""
         
        #Si abierto lo cierro.
        try:
            self.stream_out.close()
        except:
            pass
        if self.mp3:
            
            self.stream_out = self.p.open(format =
                self.p.get_format_from_width(pyaudio.paInt32),
                channels = 2,
                rate = self.mf.samplerate(),
                output = True)
        else:
            self.stream_out = self.p.open(format =
                    self.p.get_format_from_width(self.wave.getsampwidth()),
                    channels = self.wave.getnchannels(),
                    rate = self.wave.getframerate(),
                    output = True)
                
        self.online=True
        print "Stream Creado..."
    
    
    def GetInHMS(self,seconds):
        """Convierte los segundos pasados como parámetro 
        a formato HMS"""
        
        hours = seconds / 3600
        seconds -= 3600*hours
        minutes = seconds / 60
        seconds -= 60*minutes
        if hours == 0:
            return "%02d:%02d" % (minutes, seconds)
            return "%02d:%02d:%02d" % (hours, minutes, seconds)
    
    
    def tiempo(self,pos):
        """Calcula el tiempo de reproducción en función de la posición
        en la que se encuentre la lista que contiene los datos 
        del audio"""

        try:
            if self.mp3:
                if pos==0: return "00:00"
                return (self.GetInHMS( (1000+pos)*self.CHUNK/self.mf.samplerate()))
            else:
                return (self.GetInHMS(pos*self.CHUNK/self.wave.getframerate()))
        except:
            return "00:00"

    
    def get_size(self):
        """Devuelve el número de elementos de la lista de datos"""
        
        return int(len(self.data))      
        
    def get_porcentaje(self):
        """Devuelve el porcentaje que lleva reproducido, en función de
        la posición en la que se encuentre la lista"""
        if self.posicion==0:
            return 0.0
        else:		
			if float(len(self.data)) == 0:
				return 0
				print "division cero"
			else:   
				return  self.posicion/float(len(self.data)) #float para que el resultado este con decimales
        
    def play(self):
        """Maneja la acción de play"""
        self.direccion="dere"
        self.status="play"
        self.velocidad=1
        self.unlock=True
        
    def stop(self):
        """Maneja la acción de stop"""        
        self.posicion=0
        self.status="stop"
        self.direccion="dere"
        self.velocidad=1
        self.unlock=False

        
    def pause(self):      
        """Maneja la acción de pause"""
        self.status="pause"
        self.direccion="dere"
        self.velocidad=1
        self.unlock=False 
    
    def rewind(self):
        """Maneja la acción de rebobinado"""
        
        self.status="rewind"
        self.direccion="izq"
        self.velocidad=5
        self.unlock=True
                
    def forward(self):
        """Maneja la acción de reproducción rápida"""
        self.status="forward"
        self.direccion="dere"
        self.velocidad=5
        self.unlock=True

    def get_status(self):
        """Devuelve el estado en el que se encuentra el reproductor
        (play/pause...)"""
        
        return self.status
    
    def control(self):
        """Realiza el control del flujo del reproductor, es decir, 
        define las acciones que tiene que realizar ante cada cambio de
        estado"""

        if self.status=="play":
            self.play()
            if self.posicion>=len(self.data):
                self.stop()
            
        if self.status=="forward":
            self.forward()
            if self.posicion>=len(self.data):
                self.stop()
                
        if self.status=="rewind":
            self.rewind()
            if self.posicion<=0:
                self.stop()
                    
        if self.status=="stop":
            self.stop()

        if self.status=="pause":
            print "pause2"
        
    def operacion(self):
        """Determina la dirección de reproducción 
        (hacia delante/atrás)"""
        
        if self.direccion=="dere":
            self.posicion = self.posicion + self.velocidad
        else:   
            self.posicion = self.posicion - self.velocidad
            
    def run(self):        
        """Método run del hilo que se crea y que contiene el bucle 
        principal donde se encuentar la lógica principal del 
        reproductor"""

        #print "FRAMES:",self.wave.getframerate(),"CHUNK:",self.CHUNK,"ELEM:",self.get_size(),"Tiempo Total:",self.tiempo(len(self.data))
        tiempo_ant=0
        self.posicion=0
        
        while not self.fin:
            time.sleep(0.1)#para que el hilo no consuma tantos recursos
            while self.unlock and not self.fin:                
                if not self.online:                                                       
					self.data=[]
					self.p = pyaudio.PyAudio()
					self.reader()
					self.create_stream_out()  
					if not self.mp3:  
						print "FRAMES:",self.wave.getframerate(),"CHUNK:",self.CHUNK,"ELEM:",self.get_size(),"Tiempo Total:",self.tiempo(len(self.data))
					self.online=True
                self.control()
                self.stream_out.write(self.data[self.posicion] )
                
                self.operacion()
                tiempo=self.tiempo(self.posicion)

                if tiempo!=tiempo_ant:
                    tiempo_ant=tiempo
                    print  tiempo,"/",self.tiempo(len(self.data))

            if self.online:
                print "pasa a Offline"
                self.limpiar()
                data=[]
                self.online=False
                
            tiempo_ant=0
            
        self.limpiar()

    def limpiar(self):
        """Finaliza las librerías y libera los recursos bloqueados por
        el hilo"""
        try:
            self.stream_out.close()
        except:
            pass
        try:    
            self.wave.close()
        except:
            pass
        try:
            self.p.terminate()
        except:
            pass
            

    def finaliza(self):
        self.fin = True
            
def controller(data,obj):
    """Procedimiento llamado por los callbacks de los clicks de los
    botones del entorno gráfico, empleado para realizar el control 
    de las distintas opciones del reproductor (Play/Pausa/stop...)"""
    
    if data=="play":
        if obj.get_status()=="stop":
            obj.play()
        else:
            if obj.get_status()=="play":
                obj.pause()
            else:
                if obj.get_status()=="pause":
                    obj.play()      
                else:
                    if obj.get_status()=="rewind":
                        obj.play()
                    else:
                        if obj.get_status()=="forward":
                            obj.play()
                                
     
    if data=="stop":
        obj.stop()   
        
    if data=="rewind":    
        obj.rewind()

    if data=="forward":
        obj.forward()    

    print "data:",data,"estatus",obj.get_status()       

            

if __name__ == "__main__"  :
    a=audio("listening1.wav")
    while True:
        print "intro una accion",a.status
        accion=raw_input()
        if accion=="play":
            a.play()
        else:
            a.stop()
