import random

from ctypes import windll, c_buffer

class _mci:
    def __init__(self):
        self.w32mci = windll.winmm.mciSendStringA
        self.w32mcierror = windll.winmm.mciGetErrorStringA

    def send(self, command):
        buffer = c_buffer(255)
        errorcode = self.w32mci(str(command), buffer, 254, 0)
        if errorcode:
            return errorcode, self.get_error(errorcode)
        else:
            return errorcode, buffer.value

    def get_error(self, error):
        error = int(error)
        buffer = c_buffer(255)
        self.w32mcierror(error, buffer, 254)
        return buffer.value

    def directsend(self, txt):
        (err, buf) = self.send(txt)
        if err != 0:
            print 'Error %s for "%s": %s' % (str(err), txt, buf)
        return (err, buf)

# TODO: detect errors in all mci calls
class AudioClip(object):
    def __init__(self, filename):
        filename = filename.replace('/', '\\')
        self.filename = filename
        self.status="stop"
        self.mp3=None
        self._alias = 'mp3_%s' % str(random.random())

        self._mci = _mci()

        self._mci.directsend(r'open "%s" alias %s' % (filename, self._alias ))
        self._mci.directsend('set %s time format milliseconds' % self._alias)

        err, buf=self._mci.directsend('status %s length' % self._alias)
        self._length_ms = int(buf)
        self.porcentaje=5*self._length_ms /100.0

    def volume(self, level):
        """Sets the volume between 0 and 100."""
        self._mci.directsend('setaudio %s volume to %d' %
                (self._alias, level * 10) )

    def play(self, start_ms=None, end_ms=None):
        if self.status=="pause":
            self.unpause()
        elif self.status=="play":
			self.pause()
        else:
            start_ms = 0 if not start_ms else start_ms
            end_ms = self.get_size() if not end_ms else end_ms
            err,buf=self._mci.directsend('play %s from %d to %d'
                    % (self._alias, start_ms, end_ms) )
            self.status="play"

    def rewind(self):
        nueva_posicion=self.posicion()-self.porcentaje
        if nueva_posicion<0:nueva_posicion=0
        self.status="rewind"
        self.play(int(nueva_posicion))

    def forward(self):
        nueva_posicion=self.posicion() + self.porcentaje
        total=float(self._length_ms)
        if nueva_posicion>total: nueva_posicion=total -1.0
        self.status="forward"
        self.play(int(nueva_posicion))

    def get_porcentaje(self):
		if self.posicion()>=float(self._length_ms):
			self.stop()
			porcentaje=1.0
		elif self.posicion()==0.0:
			porcentaje=0.0
		else:
			porcentaje=self.posicion()/self._length_ms
		return porcentaje

    def isplaying(self):
        playing=(self._mode() == 'playing')
        if playing: 
            self.status="play"
        else:
            if self._mode()== 'stopped':self.status="stop"
        return playing
        
    def finaliza(self):
        self.stop()

    def _mode(self):
        err, buf = self._mci.directsend('status %s mode' % self._alias)
        return buf

    def posicion(self):
        err, buf = self._mci.directsend('status %s position' % self._alias)
        return float(buf)
        
    def seek(self, posicion):
        self._mci.directsend('seek %s to %s' % (self._alias,posicion))
        

    def pause(self):
        self._mci.directsend('pause %s' % self._alias)
        self.status="pause"

    def unpause(self):
        self._mci.directsend('resume %s' % self._alias)
        self.status="play"

    def ispaused(self):
        return self._mode() == 'paused'

    def stop(self):
        self._mci.directsend('stop %s' % self._alias)
        self._mci.directsend('seek %s to start' % self._alias)
        #self.seek(0)
        self.status="stop"

    def get_size(self):
        return self._length_ms

    def tiempo(self,valor):
		if hasattr(valor,'__call__'):
			valor=valor()
		minutos,milisegundos=divmod(valor,60000)
		segundos=round(valor/1000)
		if segundos >= 60: segundos=segundos- minutos*60
		s="%02i:%02i" % (minutos,segundos)
		return s
        
    # TODO: this closes the file even if we're still playing.
    # no good.  detect isplaying(), and don't die till then!
    def __del__(self):
        self._mci.directsend('close %s' % self._alias)
