#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Project:     Agencia
# Module:      speaking.py
# Purpose:     Entorno grafico para pruebas de ingles.
# Language:    Python 2.6
# Date:        15-Nov-2011.
# Ver.:        15-Nov-2011.
# Author:    Isabel Aparicio Pérez
# Copyright:    2011 - Isabel Aparicio Pérez    <prog5pe@edu.juntaextremadura.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
##############################################################################

import pango
import gtk
import pygtk
import sys
import os
import subprocess
#Módulo realizado con pyaudio para realizar la reprodución
import agencia.librep_oral as librep
import agencia.libgrab as libgrab

import platform


dir_tmp=os.path.join(os.path.expanduser("~") ,".sece")

if platform.system()=="Linux":
    os.system("preparacion_alsa.sh")

import gobject
gobject.threads_init()


    

class speaking:
    
    def crearVentana(self,titulo,X,Y):
        ##crea una ventana con el titulo y el tamaño pasados como parametros
        self.ventana= gtk.Window(gtk.WINDOW_TOPLEVEL)#
        self.ventana.set_size_request(X, Y)
        self.ventana.set_title(titulo)         
        self.ventana.show()
        self.ventana.maximize()
        return self.ventana
        
    
    def creaCaja(self,vertical,homogeneo,espacio):
        #crea una caja vertical u horizontal dependiendo del valor pasado en el argumento Vertical
        #estas cajas las vamos a usar para colocar los widgets dentro de las ventanas.
        if vertical==True:
            self.box=gtk.VBox(homogeneo,espacio)
        else:
            self.box=gtk.HBox(homogeneo,espacio)
        self.box.show()
        return self.box
    
    def creaTabla(self,filas,columnas,homogeneo):
        #crea una tabla para empaquetar widgets del tamaño filas y columnas pasados como argumentos
        self.tabla=gtk.Table(filas,columnas,homogeneo)
        self.tabla.show()
        return self.tabla
        
    def creaEtiqueta(self,texto,X,Y,alineacion,partido):
        ##se encarga de crear una etiqueta con el texto que se le pasa como parametro del 
        #tamaño X x Y con la alineacion que se le pase y tambien si realiza el particionado de
        #las lineas de forma automatica o no
        etiqueta=gtk.Label(texto)
        etiqueta.set_size_request(X,Y)
        etiqueta.set_justify(alineacion)
        etiqueta.set_line_wrap(partido)
        
        etiqueta.show()
        return etiqueta
      
    def creaBoton(self,nombre,stock,x,y):
        #crea un boton normal con el nombre y el stock predeterminado y ademas se les pasa tamaño para
        #redimensionar el boton 
        self.boton=gtk.Button(nombre,stock)
        self.boton.set_size_request(x,y)
        self.boton.show()
        return self.boton
        
    def creaCajaTexto(self,maximo,x,y,visible):
        #crea una caja que recoge un texto, podemos ponerle un tamaño maximo al texto que recoge en maximo
        #x e y son los parametros para dimensionar la caja de texto, tambien establecemos si el texto va a ser visible
        #o va a mostrar puntos
        self.entrada=gtk.Entry(maximo)
        self.entrada.set_size_request(x,y)
        self.entrada.set_visibility(visible)
        self.entrada.show()
        return self.entrada
        
    def creaCajaBotones(self,horizontal,disposicion,espacio):
        #creamos una barra de botones horizontal o vertical dependiendo del valor del argumento horizontal
        #y ademas le pasamos el parametro disposicion que nos va a indicar el estilo de la barra de botones
        if horizontal==True:
            self.cajaboton=gtk.HButtonBox()
        else:
            self.cajaboton=gtk.VButtonBox()
        self.cajaboton.set_layout(disposicion)
        self.cajaboton.set_spacing(espacio)
        self.cajaboton.show()
        return self.cajaboton
        
    def creaFrame(self,titulo,estilo):
        #crea un marco con el titulo y el estilo pasados por parametros
        self.frame=gtk.Frame(titulo)
        self.frame.set_shadow_type(estilo)
        self.frame.show()
        return self.frame
        
    def creaImagen(self,ruta):
        self.imagen = gtk.Image()
        self.imagen.set_from_file(ruta)
        self.imagen.show()
        return self.imagen
        
    def cerrarVentana(self,widget,data):
        #con esto se cierra la ventana activa
        self.ventana.destroy()
                
    def borradoevento(self,widget=None,data=None):
        ##pregunta antes de cerrar la ventana, si le decimos que no no cierra el dialogo
        texto="   ¿Está seguro de que desea salir de la aplicación?" 
        dialogo=gtk.MessageDialog(self.ventana, gtk.DIALOG_MODAL,gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO,texto)        
        dialogo.show() 
        respuesta = dialogo.run()
        dialogo.destroy()
        if respuesta == gtk.RESPONSE_YES:
            dialogo.destroy()
            self.ventana.destroy()
            destruccion=False
        else:
            dialogo.destroy()
            destruccion=True
        return destruccion        
        
    def destruir(self,widget,data):
        #cierra el programa
        gtk.main_quit()
        
    def destruirRepGrab(self,widget,data,rep,gra):
        #cierra el programa
        gra.fin=True
        print gra.fin
        rep.finaliza()
        gra.finaliza()
        gtk.main_quit()

    def muestraMensaje (self,widget,data):
        dialog = gtk.MessageDialog (self.ventana,
                                       gtk.DIALOG_DESTROY_WITH_PARENT,
                                       gtk.MESSAGE_INFO,
                                       gtk.BUTTONS_CLOSE,
                                       data);
        dialog.show()
        dialog.run()
        dialog.destroy()
        
        
    def ayudar(self,widget,data):
        ##boton de ayuda de la ventana 
        ventanaayuda=self.crearVentana("Ayuda",700,500)
        color = gtk.gdk.Color(red=60000, green=60000, blue=20000, pixel=0)
        ventanaayuda.modify_bg(gtk.STATE_NORMAL, color)
        caja=self.creaCaja(True,True,10)
        ventanaayuda.add(caja)
        texto="  El programa comenzará pidiéndole una grabación de 15 segundos," +\
        " tras los cuales, podrá escuchar lo que se ha grabado para comprobar" +\
        " que todo funciona correctamente.\r\r"                                       +\
        " Seguidamente podrá repetir la comprobación o comenzar la prueba. \r"       +\
        " \r Debe tener en cuenta que una vez comience la prueba no habrá opción a repetirla."
        
        
       

        etiquetaayuda=self.creaEtiqueta(texto,700,500,gtk.JUSTIFY_FILL,True)
        
        pango1 = pango.FontDescription("15")

        etiquetaayuda.modify_font(pango1)
        
    
        caja.pack_start(etiquetaayuda,False,False,10)
        
        
        #cajaboton=self.creaCajaBotones(True,gtk.BUTTONBOX_SPREAD,1)
        cajaboton=self.creaCajaBotones(True,gtk.BUTTONBOX_EDGE,1)
        botonsalir=self.creaBoton(None,gtk.STOCK_CLOSE,30,30)
        
        cajaboton.add(botonsalir) 
          
        caja.pack_start(cajaboton,True,True,20)
        botonsalir.connect("clicked",self.cerrarVentana,None)
        
        
        
    def compruebaNia(self,widget,data):
        p1 = self.contra1.get_text()
        p2 = self.contra2.get_text()
        
        if p1 != p2:
            self.muestraMensaje(self.ventanaclave,"Los nombres de archivo no coinciden.")
        else:
            if p1=="":
                self.muestraMensaje(self.ventanaclave,"El nombre no puede estar en blanco")                                  
            else:
                self.ventanaclave.hide()            
                self.botonplay.hide()
                self.botonrepetir.hide()
                self.botonexamen.hide()
                self.barraprogreso.hide()
                
                self.botongrab.show()
                self.barrasonido_left.show() 
                self.barrasonido_right.show()
                self.barraprog_grab.show()

                texto = "  Pulsa el botón con el círculo rojo para iniciar la grabación.\r\r" +\
                "  Cuando quieras terminar la prueba, pulsa el botón del círculo rojo."
            
                self.etiqueta.set_label(texto)
                self.marco.set_label("Prueba de Evaluación")
           
                #self.grab.fin=True
                #self.grab=libgrab.recorder("record1.wav",14)
            
                #self.controlador("repetir",self.grab)

            
                p1=p1.strip()
            
                self.grab.fichero = p1 + ".wav"
                self.grab.set_tiempo_grab(300)
            
                self.ventanareproductor.show()
                #self.comenzado = False
                #self.proc_barra_grab()
            
           
    
    def pedirNia(self):        
        self.ventanaclave=self.crearVentana("Nombre indentificativo de fichero.",430,250)
        tabla=self.creaTabla(9,8,True)
        self.ventanaclave.add(tabla)
        texto= "Introduzca el nombre que vamos a dar al fichero:"
        etiqueta1=self.creaEtiqueta(texto,400,30,gtk.JUSTIFY_FILL,True)        
        
        tabla.attach(etiqueta1,1,7,1,5,True,True,0)
        self.contra1=self.creaCajaTexto(20,300,30,True)
        tabla.attach(self.contra1,1,7,3,4,True,True,0)
        
        texto= "Repita el nombre:"
        etiqueta2=self.creaEtiqueta(texto,300,100,gtk.JUSTIFY_CENTER,True)        
        tabla.attach(etiqueta2,1,7,2,7,True,True,0)
        self.contra2=self.creaCajaTexto(20,300,30,True)
        tabla.attach(self.contra2,1,7,5,6,True,True,0)
        barrabotones=self.creaCajaBotones(True,gtk.BUTTONBOX_SPREAD,30)
        tabla.attach(barrabotones,0,8,7,8,True,True,0)  
        botonok=self.creaBoton(None,gtk.STOCK_OK,30,30)
        barrabotones.add(botonok)
        
        botonok.connect("clicked",self.compruebaNia,None)
        #botonayuda=self.creaBoton(None,gtk.STOCK_HELP,30,30)
        #barrabotones.add(botonayuda)
        #botonayuda.connect("clicked",self.ayudar,None)
        self.ventanaclave.connect("delete_event",self.borradoevento)
        self.ventanaclave.connect("destroy",self.destruir,None)
        
        

               
    
    def grabador(self,widget,data):
        
        ####AQUI HABRA QUE LLAMAR A UNA FUNCION QUE HAGA LAS COMPROBACIONES DE LOS 
        #####LOGIN, COMPROBARA QUE SEA IGUAL, DISCRIMINARA SI SE HA INTRODUCIDO
        #####LA CONTRASEÑA DE PRUEBA O LA CONTRASEÑA DE EXAMEN O SI HA HABIDO ERROR.
        #####EN CASO DE QUE SE PRODUZCA ERROR DEBE VOLVER A LA FUNCION PEDIRCLAVE.
        ######EN CASO DE QUE HAYA INTRODUCIDO LA CLAVE CORRECTA QUE DEVUELVA EL FICHERO
        #####QUE HAY QUE TRATAR Y DEVUELVA EL CONTROL A ESTA FUNCION.
        self.ventanappal.hide()
      
        self.ventanareproductor=self.crearVentana("Prueba de expresión oral",800,500)
        tabla=self.creaTabla(7,8,True)
        #self.marco=self.creaFrame("Agencia Extremeña de Evaluación - Prueba de Grabación ",gtk.SHADOW_IN)
        self.marco1=self.creaFrame("Servicio de Evaluación y Calidad Educativa",gtk.SHADOW_IN)
        
        self.marco=self.creaFrame("Prueba de Grabación ",gtk.SHADOW_IN)
        #tabla.attach(self.marco,1,7,1,2,True,True,0)
        
        ##en esta etiqueta en vez de el texto lo que tendremos que poner sera el nombre del fichero
        ##que se este reproduciendo en el momento pero por ahora pongo un texto
        texto = "  Vamos a realizar una prueba de sonido. \r\r  Pulsa el botón con el círculo rojo  y comienza a hablar.\r\r" +\
        "Dispones de 15 segundos; después veremos si ha funcionado."
        
        
        barraVertical=self.creaCajaBotones(True,gtk.BUTTONBOX_SPREAD,2)
        

        image = gtk.Image()
        pixbuf = gtk.gdk.pixbuf_new_from_file("./iconos/edexpresion.jpg")
        scaled_buf = pixbuf.scale_simple(175,175,gtk.gdk.INTERP_BILINEAR)
        image.set_from_pixbuf(scaled_buf)
        image.show()
        
        self.marco1.add(image)        
        barraVertical.add(self.marco1)
        barraVertical.add(self.marco)
        
        
        #tabla.attach(self.marco,1,7,1,2,True,True,0)
        
        self.etiqueta=self.creaEtiqueta(texto,400,200,gtk.JUSTIFY_FILL,True)
        self.marco.add(self.etiqueta)
        
        tabla.attach(barraVertical,1,7,1,2,True,True,0)
        
        self.ventanareproductor.add(tabla)
        
        
        
        
        
        ##Creamos la barra de progreso de avance de la reproducción
        self.barraprogreso = gtk.ProgressBar(adjustment=None)
        self.barraprogreso.set_usize(450, 40)
         
        self.barraprog_grab = gtk.ProgressBar(adjustment=None)
        self.barraprog_grab.set_usize(450, 40)
        self.barrasonido_left = gtk.ProgressBar(adjustment=None)
        self.barrasonido_right = gtk.ProgressBar(adjustment=None)


        tabla.attach(self.barraprogreso,1,7,6,7,True,True,0)
        tabla.attach(self.barraprog_grab,1,7,1,7,True,True,0)
        tabla.attach(self.barrasonido_left,1,7,5,7,True,True,0)
        tabla.attach(self.barrasonido_right,1,7,6,7,True,True,0)
        
        
       
        self.barraprog_grab.show()
        self.barrasonido_left.show()
        self.barrasonido_right.show()
       
        
        ##creamos una barra de botones y añadimos los botones correspondientes al reproductor
        barrabotones=self.creaCajaBotones(True,gtk.BUTTONBOX_SPREAD,10)
        
        #Creamos instancia de la clase audio.
        #self.repro=librep.audio("record1.wav")
        self.repro=librep.audio()
        self.grab=libgrab.recorder("record1.wav",15)
        
        self.comenzado = False
        self.examen=False
        self.proc_barra_grab()

   
        # ****** BOTON_ATRAS
        imagenatras = self.creaImagen("./iconos/atras.png")
        botonatras = gtk.Button()
        botonatras.add(imagenatras)
        #self.botonatras.connect("clicked",self.clickenatras)
        botonatras.connect("clicked", self.callback, self.repro, "rewind")
        #botonatras.show()
        barrabotones.add(botonatras)
        
        # ****** BOTON GRABAR
        imagenplay = self.creaImagen("./iconos/grabar.png")
        self.botongrab = gtk.Button()
        self.botongrab.add(imagenplay)
        #self.botonplay.connect("clicked",self.clickenplay_pausa)
        self.botongrab.connect("clicked", self.callback, self.grab, "record")
        self.botongrab.show()
        barrabotones.add(self.botongrab)

        # ****** BOTON PLAY
        
        #imagenplay = self.creaImagen("./iconos/play.png")
        #self.botonplay = gtk.Button()
        #self.botonplay.add(imagenplay)
        #self.botonplay.connect("clicked",self.clickenplay_pausa)

        self.botonplay=self.creaBoton(None,"Escucha lo que has grabado",185,30)
        self.botonplay.connect("clicked", self.callback, self.repro, "play")
        self.botonplay.hide()    
        barrabotones.add(self.botonplay)
        
        # ****** BOTON REPETIR PRUEBA
        self.botonrepetir=self.creaBoton(None,"Repetir comprobación",175,30)
        barrabotones.add(self.botonrepetir)
        self.botonrepetir.hide()
        self.botonrepetir.connect("clicked", self.callback, self.grab, "repetir")
        
        # ****** BOTON REALIZAR EXÁMEN
        self.botonexamen=self.creaBoton(None,"NO TOCAR HASTA QUE AVISE EL PROFESOR",350,30)
        
       
        #Cambio gdk.color a rojo
        
        map = self.botonexamen.get_colormap() 
        color = map.alloc_color("red")

        #Copio el estilo actual and Reemplazo el fondo actual.
        style = self.botonexamen.get_style().copy()
        style.bg[gtk.STATE_NORMAL] = color

        #Asigna el nuevo estilo
        self.botonexamen.set_style(style)

        
        
        barrabotones.add(self.botonexamen)
        self.botonexamen.hide()
        self.botonexamen.connect("clicked", self.callback, self.grab, "examen")
        
        
        
        """
        
        # ****** BOTON SIGUIENTE
        imagensiguiente = self.creaImagen("./iconos/siguiente.png")
        botonsiguiente = gtk.Button()
        botonsiguiente.add(imagensiguiente)
        #self.botonsiguiente.connect("clicked",self.clickensiguiente)
        botonsiguiente.connect("clicked", self.callback, self.repro, "forward")
        #botonsiguiente.show()
        barrabotones.add(botonsiguiente)

        # ****** BOTON STOP
        imagenstop = self.creaImagen("./iconos/stop.png")
        self.botonstop = gtk.Button()
        self.botonstop.add(imagenstop)
        #self.botonstop.connect("clicked",self.clickenstop)
        self.botonstop.connect("clicked", self.callback, self.repro, "stop")
        #botonstop.show()
        barrabotones.add(self.botonstop)
        """
 
        tabla.attach(barrabotones,1,7,4,6,True,True,0)
        
        self.ventanareproductor.connect("delete_event",self.borradoevento)
        self.ventanareproductor.connect("destroy",self.destruirRepGrab,None,self.repro,self.grab)
        
        
        

    def controlador(self,accion,rep):
        if accion == 'play':
            #self.barraprog_grab.hide()
            
            self.botonrepetir.set_sensitive(False)
            self.botonexamen.set_sensitive(False)
            self.botonplay.set_sensitive(False)
            
            
            librep.controller(accion,rep)
            self.proc_barra()
            
            
        
        if accion == 'stop':
            if self.comenzado:
                self.comenzado = False
                self.botongrab.set_sensitive(True)
                if self.examen:
                    self.muestraMensaje(self.ventana, "Prueba Terminada")
                else:
                    self.muestraMensaje(self.ventana, "Comprobación terminada  \nPulsa aquí para probar.")
                    
                self.repro.ponerFichero('record1.wav')                
                self.proc_barra()
                
                
                

            self.botongrab.hide()
            self.barraprog_grab.hide()
            self.barrasonido_left.hide()
            self.barrasonido_right.hide()
            self.barraprogreso.show()          
            self.botonplay.show()
            self.botonrepetir.show()
            self.botonexamen.show()
            texto="  Hemos realizado la comprobación. No toques el botón Rojo hasta que te avise el profesor.\r\rAhora puedes: \r-Pulsar el Botón 'Escucha lo que has grabado' \r-Repetir comprobación si no se oye nada o se escucha muy mal."
            self.etiqueta.set_label(texto)
            
            self.controlador('play',self.repro)
            
            #librep.controller('play',self.repro)
            
        if accion == 'repetir':
            texto = "  Vamos a realizar una prueba de sonido. \r\r  Pulsa el botón con el círculo rojo y comienza a hablar.\r\r" +\
            "Dispones de 15 segundos; después veremos si ha funcionado."
            self.etiqueta.set_label(texto)
            self.botonplay.hide()
            self.botonrepetir.hide()
            self.botonexamen.hide()
            self.barraprogreso.hide()
            
            self.botongrab.show()
            self.barrasonido_left.show() 
            self.barrasonido_right.show()
            self.barraprog_grab.show()
            self.comenzado = False
            
              

        if accion == 'record':
            if self.comenzado == False:
                self.comenzado = True
                rep.status="grabar"
                if self.examen:
                    self.botongrab.set_sensitive(True)                   
                else:
                    self.botongrab.set_sensitive(False)
                            
            else:
                rep.status="pausa"
                print "Entra aquí " + rep.status
                self.ventanareproductor.hide()
                texto="   ¿Seguro que quieres terminar la prueba?" 
                dialogo=gtk.MessageDialog(self.ventana, gtk.DIALOG_MODAL,gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO,texto)        
                dialogo.show() 
                respuesta = dialogo.run()
                dialogo.destroy()
                if respuesta == gtk.RESPONSE_YES:
                    dialogo.destroy()
                    
                    destruccion=False
                    rep.fin=True 
                                        
                else:
                    rep.status="grabar"
                    dialogo.destroy()
                    destruccion=True
                    self.ventanareproductor.show()
 
        if accion == 'examen':
            #Crear Ventana de aviso
            texto=" ¿Estás seguro de que quieres empezar?, una vez que comience no hay opción a repetir." 
            dialogo=gtk.MessageDialog(self.ventana, gtk.DIALOG_MODAL,gtk.MESSAGE_ERROR, gtk.BUTTONS_YES_NO,texto)
            dialogo.show() 
            respuesta = dialogo.run()
            dialogo.destroy()           
            if respuesta == gtk.RESPONSE_YES:
                dialogo.destroy()
                destruccion=True
                self.examen=True
                self.ventanareproductor.hide()
                self.pedirNia()
               
                
            else:
                dialogo.destroy()
                destruccion=True

            
        
    #Función para llamar a las acciones de cada botón del reproductor    
    def callback(self, widget, rep, data):
        if data == 'play':
            self.controlador('play',rep)            
       
        if data == 'record':
            self.controlador('record',rep)
           
        if data == 'repetir':
            self.controlador('repetir',rep)
 
        if data == "stop":
            self.controlador('stop',rep)
            

        if data == "examen":            
            self.controlador('examen',rep)
            
            
   
        


        
    def proc_barra_grab(self):
        #self.barraprog_grab.set_fraction(self.grab.get_porcentaje())
        estado = self.grab.get_status()
        self.barraprog_grab.set_fraction(self.grab.get_porcentaje())
        vol=self.grab.get_vol(self.grab.data_inst)
        if vol<0 or vol>1: vol=1
        
        self.barrasonido_left.set_fraction(vol)
        self.barrasonido_right.set_fraction(vol)
        #self.barrasonido_left.set_fraction(self.grab.get_vol(self.grab.data_inst))
        #self.barrasonido_right.set_fraction(self.grab.get_vol(self.grab.data_inst))
        
        text=str(self.grab.tiempo(self.grab.posicion)) + "/" + str(self.grab.tiempo(self.grab.get_size())) #+"----"+str(rep.get_porcentaje())
        #print self.grab.posicion        
        self.barraprog_grab.set_text(text)
       
        
        if self.grab.status=="stop" and self.comenzado:
            self.controlador('stop','')
            self.repro.ponerFichero('record1.wav')
            if self.examen:
                print "Debe Terminar"
                self.ventanareproductor.destroy()
                #Finaliza el programa.  
                
        #if  self.grab.tiempo(self.grab.posicion) > "00:01" :
        #    self.botongrab.set_sensitive(True)
        

        gobject.timeout_add(100, self.proc_barra_grab)
        
        
    def proc_barra(self):
        self.barraprogreso.set_fraction(self.repro.get_porcentaje())
        text=str(self.repro.tiempo(self.repro.posicion))+ "/" +  str(self.repro.tiempo(self.repro.get_size())) #+"----"+str(rep.get_porcentaje())
        self.barraprogreso.set_text(text)
        if self.repro.status == "stop":
            self.repro.stop()
            self.botonrepetir.set_sensitive(True)
            self.botonexamen.set_sensitive(True)
            self.botonplay.set_sensitive(True)
        
        gobject.timeout_add(100, self.proc_barra)   
    
    def __init__(self):
        self.ventanappal=self.crearVentana("Prueba de expresión oral",300,300)
        tabla=self.creaTabla(6,6,False)
        self.ventanappal.add(tabla)
        
        #barraInicio=self.creaCajaBotones(False,gtk.BUTTONBOX_SPREAD,30)
        barraInicio=self.creaCajaBotones(False,gtk.BUTTONBOX_EDGE,30)
        
        botonentrar=self.creaBoton("Entrar",None,20,50)
        botonentrar.connect("clicked",self.grabador,None)
        
        #tabla.attach(botonentrar,1,7,1,3,True,True,0)
        
        image = gtk.Image()
        pixbuf = gtk.gdk.pixbuf_new_from_file("./iconos/edexpresion.jpg")
        scaled_buf = pixbuf.scale_simple(400,400,gtk.gdk.INTERP_BILINEAR)
        image.set_from_pixbuf(scaled_buf)
        image.show()

        tabla.attach(image,2,3,1,3,True,True,0)
        
        botonayuda=self.creaBoton(None,gtk.STOCK_HELP,150,50)
        botonayuda.connect("clicked",self.ayudar,None)
        
        #tabla.attach(botonayuda,1,7,2,4,True,True,0)
        
        
       

        #barraInicio.add(image)
        barraInicio.add(botonentrar)
        barraInicio.add(botonayuda)
        
        tabla.attach(barraInicio,1,7,5,6,True,True,0)

        self.ventanappal.connect("delete_event",self.borradoevento)
        self.ventanappal.connect("destroy",self.destruir,None)
        
            
    def main(self):
        gtk.main()

if __name__ == "__main__":
    base = speaking()
    base.main()
